#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <conio.h>
#include <windows.h>
#include "inout.h" // header file generated by MIDL compiler

__int64 Time;

void StartTime( void )
{
	QueryPerformanceCounter( (LARGE_INTEGER *) &Time );
}

ULONG EndTime( void )
{
	__int64 liDiff;
	__int64 liFreq;

	QueryPerformanceCounter( (LARGE_INTEGER *) &liDiff );

	liDiff -= Time;
	liDiff *= 1000; // Adjust to milliseconds, shouldn't overflow...

	QueryPerformanceFrequency( (LARGE_INTEGER *) &liFreq );

	return (ULONG) ( liDiff / liFreq );
}

void Usage( char * pszProgramName )
{
	fprintf( stderr, "Usage: %s\n", pszProgramName );
	fprintf( stderr, "-a use_authn [range 0..6, def 0; Win95, only 0..2]\n" );
	fprintf( stderr, "-b auth_name auth_domain auth_pwd [def current login context]\n" );
	fprintf( stderr, "-d principal_name [def NULL]\n" );
	fprintf( stderr, "-p protocol_sequence [def ncalrpc]\n" );
	fprintf( stderr, "-n network_address [def ""]\n" );
	fprintf( stderr, "-e endpoint [def foo]\n" );
	fprintf( stderr, "-o options\n" );
	fprintf( stderr, "-s string_size (0 varies) [def 1024]\n" );
	fprintf( stderr, "-c array_count [def 16]\n" );
	fprintf( stderr, "-i iteration_count [def 0]\n" );
	fprintf( stderr, "-ping call ping() [empty function] iteration_count times\n" );
	fprintf( stderr, "-ex call raiseex() [test function] once and exit\n" );
	return;
}

int _CRTAPI1 main( int argc, char **argv )
{
	RPC_STATUS status;
	handle_t binding;
	unsigned char * pszUuid = NULL;
	unsigned char * pszProtocolSequence = "ncalrpc";
	unsigned char * pszNetworkAddress = NULL;
	unsigned char * pszEndpoint = "foo";
	unsigned char * pszOptions = NULL;
	unsigned char * pszStringBinding = NULL;
	char *defprincname = "midlbug";
	char errtxt[4096];
	int useprincname = 0;
	int DoShutdown = 0;
	long size = 1024L;
	long count = 16L;
	long iters = 0L;
	int useauthn = 0;
	int pingflag = 0, exflag = 0;
	double GotKB;
	int i, j;
	unsigned long lastsec = 0L;
	SEC_WINNT_AUTH_IDENTITY ident;
	int useident = 0;

	memset( &ident, '\0', sizeof ident );
	/* allow the user to override settings with command line switches */
	for ( i = 1; i < argc; i ++ )
	{
		if ( *argv[i] == '-' || *argv[i] == '/' )
		{
			switch ( tolower( argv[i][1] ) )
			{
				case 'p':  // protocol sequence or ping
					if ( stricmp( &argv[i][1], "ping" ) == 0 )
						pingflag = 1;
					else
						pszProtocolSequence = argv[++ i];
					break;
				case 'n':  // network address
					pszNetworkAddress = argv[++ i];
					break;
				case 'e':
					if ( stricmp( &argv[i][1], "ex" ) == 0 )
						exflag = 1;
					else
						pszEndpoint = argv[++ i];
					break;
				case 'o':
					pszOptions = argv[++ i];
					break;
				case 'd':
					defprincname = argv[++ i];
					useprincname = 1;
					break;
				case 'a':
					useauthn = atoi( argv[++ i] );
					if ( useauthn < 1 || useauthn > 6 )
					{
						fprintf( stderr, "%ld: illegal useauthn.\n", useauthn );
						useauthn = 0;
					}
					break;
				case 'b':
					useident = 1;
					ident.User = argv[++ i];
					ident.Domain = argv[++ i];
					ident.Password = argv[++ i];
					ident.UserLength = strlen( ident.User );
					ident.DomainLength = strlen( ident.Domain );
					ident.PasswordLength = strlen( ident.Password );
					ident.Flags = SEC_WINNT_AUTH_IDENTITY_ANSI;
					break;
				case 's':
					size = atoi( argv[++ i] );
					if ( size < 0 || size > 4096 * 1024 )
					{
						fprintf( stderr, "%ld: illegal string size.\n", size );
						size = 1024;
					}
					break;
				case 'c':
					count = atoi( argv[++ i] );
					if ( count < 1 || count > 65536 )
					{
						fprintf( stderr, "%ld: illegal count.\n", count );
						count = 1;
					}
					break;
				case 'i':
					iters = atoi( argv[++ i] );
					if ( iters < 0 )
					{
						fprintf( stderr, "%ld: illegal iteration count.\n", iters );
						iters = 0;
					}
					break;
				case 'h':
				case '?':
				default:
					Usage( argv[0] );
					return 1;
				}
		}
		else
		{
			Usage( argv[0] );
			return 1;
		}
	}

	/* Use a convenience function to concatenate the elements of */
	/* the string binding into the proper sequence. */
	status = RpcStringBindingCompose( pszUuid, pszProtocolSequence,
	  pszNetworkAddress, pszEndpoint, pszOptions, &pszStringBinding );
	printf( "RpcStringBindingCompose returned 0x%x\n", status );
	printf( "pszStringBinding = %s\n", pszStringBinding );
	if ( status )
		return status;

	/* Set the binding handle that will be used to bind to the server. */
	status = RpcBindingFromStringBinding( pszStringBinding, &binding );
	printf( "RpcBindingFromStringBinding returned 0x%x\n", status );
	if ( status )
		return status;

	if ( useauthn )
	{
		char *princname = NULL;

		if ( useident )
			printf( "Using ident: user {%s}, domain {%s}, pwd {%s}\n",
			  ident.User, ident.Domain, ident.Password );
		else
			puts( "Using default login context" );

		if ( useprincname )
			princname = defprincname;

		printf( "Using principal name {%s}\n", princname == NULL? "*NULL*": princname );
		status = RpcBindingSetAuthInfo( binding, princname,
		  useauthn, RPC_C_AUTHN_WINNT, useident? &ident: NULL, 0 );
	}

	printf( "Calling the remote procedure: iters = %ld, count = %ld, "
	  "size = %ld\n\n", iters, count, size );

	RpcTryExcept
	{
		a_t *w;
		double diff;

		StartTime();
		i = 0;
		if ( exflag )
		{
			long rc;
			error_status_t e;
			rc = raiseex( binding, &e );
			printf( "raiseex() returned %ld [%lxh], e == %ld [%lxh]\n", rc, rc, (long) e, (long) e );
			goto cleanup;
		}
		else if ( pingflag )
		{
			while ( 1 )
			{

				// check for abort
				if ( kbhit() )
				{
					while ( kbhit() )
					{
						if ( getch() == 27 ) // on Escape, shutdown server
							DoShutdown = 1;
					}
					break;
				}
				if ( iters != 0 && i >= iters )
					break;
				i ++;

				// call server
				ping( binding );

				// check if we should update progress line
				if ( lastsec != EndTime() / 1000 )
				{ // one second passed? --> time for screen update
					lastsec = EndTime();
					printf( "\r%d, %u:%02u:%02u.%03u secs ", i, lastsec / 3600000,
					  ( lastsec % 3600000 ) / 60000, ( lastsec % 60000 ) / 1000,
					  lastsec % 1000 );
					lastsec /= 1000;
				}
			}
		}
		else
		{
			while ( 1 )
			{

				// check for abort
				if ( kbhit() )
				{
					while ( kbhit() )
					{
						if ( getch() == 27 ) // on Escape, shutdown server
							DoShutdown = 1;
					}
					break;
				}
				if ( iters != 0 && i >= iters )
					break;
				i ++;

				// call server, free garbage data
				w = NULL;
				inout4( binding, size, count, &w );
				if ( w != NULL )
				{
					for ( j = 0; j < w->len; j ++ )
					{
						if ( w->a[j].t == e_string && w->a[j].vu.u_string != NULL )
						{
							midl_user_free( w->a[j].vu.u_string );
						}
					}
					midl_user_free( w );
				}

				// check if we should update progress line
				if ( lastsec != EndTime() / 1000 )
				{ // one second passed? --> time for screen update
					lastsec = EndTime();
					printf( "\r%d, %u:%02u:%02u.%03u secs ", i, lastsec / 3600000,
					  ( lastsec % 3600000 ) / 60000, ( lastsec % 60000 ) / 1000,
					  lastsec % 1000 );
					lastsec /= 1000;
				}
			}
		} // else pingflag

		lastsec = EndTime();
		diff = (double) lastsec / 1000.0;
		printf( "\r%ld calls in %d:%02d:%02d.%.03d secs\n", (long) i,
		  lastsec / 3600000, ( lastsec % 3600000 ) / 60000,
		  ( lastsec % 60000 ) / 1000, lastsec % 1000 );
		if ( !pingflag )
		{
			if ( size != 0 )
				size *= count; // account for # array elements
			else
				size = count * ( count + 1 ) / 2; // ascending sizes
			GotKB = (double) i * (double) size / 1024.0;
			printf( "%.2f KB (%.2f MB) total, %.2lf KB/sec\n",
			  GotKB, GotKB / 1024.0, GotKB / diff );
		}
		else
			size = 0;
		printf( "%.2lf msecs/call, %.2lf calls/sec, %.2f KB/call\n\n",
		  diff * 1000.0 / i, i / diff, (double) size / 1024.0 );
		if ( DoShutdown )
			Shutdown( binding );
	}
	RpcExcept( 1 )
	{
		DceErrorInqText( RpcExceptionCode(), errtxt );
		printf( "\nRuntime reported exception %ld:\n  %s\n", RpcExceptionCode(), errtxt );
	}
	RpcEndExcept

cleanup:
	/* The call to the remote procedure is complete. */
	/* Free the string and binding handle. */
	status = RpcBindingFree( &binding );
	printf( "RpcBindingFree returned 0x%x\n", status );
	if ( status )
		return status;

	status = RpcStringFree( &pszStringBinding );
	printf( "RpcStringFree returned 0x%x\n", status );
	if ( status )
		return status;

	return 0;

}


/*********************************************************************/
/*				 MIDL allocate and free							*/
/*********************************************************************/

void __RPC_FAR * __RPC_USER midl_user_allocate( size_t len )
{
	return malloc( len );
}

void __RPC_USER midl_user_free( void __RPC_FAR * ptr )
{
	free( ptr );
}

/* end file inoutc.c */
